@@ -32,6 +32,7 @@
 #include "ephy-encodings.h"
 #include "ephy-file-helpers.h"
 #include "ephy-filters-manager.h"
+#include "ephy-flatpak-utils.h"
 #include "ephy-history-service.h"
 #include "ephy-profile-utils.h"
 #include "ephy-settings.h"
@@ -42,7 +43,6 @@
 #include "ephy-web-app-utils.h"
 #include "ephy-web-extension-proxy.h"
 
-#include <dazzle.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <stdlib.h>
@@ -103,7 +103,7 @@ static EphyEmbedShell *embed_shell = NULL;
 
 static void ephy_embed_shell_tabs_catalog_iface_init (EphyTabsCatalogInterface *iface);
 
-G_DEFINE_TYPE_WITH_CODE (EphyEmbedShell, ephy_embed_shell, DZL_TYPE_APPLICATION,
+G_DEFINE_TYPE_WITH_CODE (EphyEmbedShell, ephy_embed_shell, GTK_TYPE_APPLICATION,
                          G_ADD_PRIVATE (EphyEmbedShell)
                          G_IMPLEMENT_INTERFACE (EPHY_TYPE_TABS_CATALOG,
                                                 ephy_embed_shell_tabs_catalog_iface_init))
@@ -985,10 +985,6 @@ ephy_embed_shell_startup (GApplication *application)
 
   G_APPLICATION_CLASS (ephy_embed_shell_parent_class)->startup (application);
 
-  filters_dir = adblock_filters_dir (shell);
-  priv->filters_manager = ephy_filters_manager_new (filters_dir);
-  g_free (filters_dir);
-
   ephy_embed_shell_create_web_context (shell);
 
   ephy_embed_shell_setup_web_extensions_server (shell);
@@ -1090,6 +1086,10 @@ ephy_embed_shell_startup (GApplication *application)
   ephy_embed_prefs_set_cookie_accept_policy (cookie_manager, cookie_policy);
   g_free (cookie_policy);
 
+  filters_dir = adblock_filters_dir (shell);
+  priv->filters_manager = ephy_filters_manager_new (filters_dir);
+  g_free (filters_dir);
+
 #if ENABLE_HTTPS_EVERYWHERE
     /* We might want to be smarter about this in the future. For now,
      * trigger an update of the rulesets once each time Epiphany is started.
@@ -1567,6 +1567,10 @@ ephy_embed_shell_launch_handler (EphyEmbedShell *shell,
   g_assert (EPHY_IS_EMBED_SHELL (shell));
   g_assert (file || mime_type);
 
+  if (ephy_is_running_inside_flatpak ()) {
+    return ephy_file_launch_file_via_uri_handler (file);
+  }
+
   app = ephy_file_launcher_get_app_info_for_file (file, mime_type);
 
   /* Do not allow recursive calls into the browser, they can lead to
