@@ -391,23 +391,6 @@ ephy_history_service_commit_transaction (EphyHistoryService *self)
   }
 }
 
-static void
-ephy_history_service_enable_foreign_keys (EphyHistoryService *self)
-{
-  GError *error = NULL;
-
-  if (self->history_database == NULL)
-    return;
-
-  ephy_sqlite_connection_execute (self->history_database,
-                                  "PRAGMA foreign_keys = ON", &error);
-
-  if (error) {
-    g_warning ("Could not enable foreign keys pragma: %s", error->message);
-    g_error_free (error);
-  }
-}
-
 static gboolean
 ephy_history_service_open_database_connections (EphyHistoryService *self)
 {
@@ -419,8 +402,9 @@ ephy_history_service_open_database_connections (EphyHistoryService *self)
     g_object_unref (self->history_database);
 
   self->history_database = ephy_sqlite_connection_new (self->read_only ? EPHY_SQLITE_CONNECTION_MODE_READ_ONLY
-                                                                       : EPHY_SQLITE_CONNECTION_MODE_READWRITE);
-  ephy_sqlite_connection_open (self->history_database, self->history_filename, &error);
+                                                                       : EPHY_SQLITE_CONNECTION_MODE_READWRITE,
+                                                       self->history_filename);
+  ephy_sqlite_connection_open (self->history_database, &error);
   if (error) {
     g_object_unref (self->history_database);
     self->history_database = NULL;
@@ -434,10 +418,10 @@ ephy_history_service_open_database_connections (EphyHistoryService *self)
     }
     g_error_free (error);
     return FALSE;
+  } else {
+    ephy_sqlite_connection_enable_foreign_keys (self->history_database);
   }
 
-  ephy_history_service_enable_foreign_keys (self);
-
   return self->read_only ||
           (ephy_history_service_initialize_hosts_table (self) &&
            ephy_history_service_initialize_urls_table (self) &&
@@ -691,8 +675,8 @@ ephy_history_service_add_visit (EphyHistoryService *self, EphyHistoryPageVisit *
 {
   EphyHistoryServiceMessage *message;
 
-  g_return_if_fail (EPHY_IS_HISTORY_SERVICE (self));
-  g_return_if_fail (visit != NULL);
+  g_assert (EPHY_IS_HISTORY_SERVICE (self));
+  g_assert (visit != NULL);
 
   message = ephy_history_service_message_new (self, ADD_VISIT,
                                               ephy_history_page_visit_copy (visit),
@@ -706,8 +690,8 @@ ephy_history_service_add_visits (EphyHistoryService *self, GList *visits, GCance
 {
   EphyHistoryServiceMessage *message;
 
-  g_return_if_fail (EPHY_IS_HISTORY_SERVICE (self));
-  g_return_if_fail (visits != NULL);
+  g_assert (EPHY_IS_HISTORY_SERVICE (self));
+  g_assert (visits != NULL);
 
   message = ephy_history_service_message_new (self, ADD_VISITS,
                                               ephy_history_page_visit_list_copy (visits),
@@ -721,7 +705,7 @@ ephy_history_service_find_visits_in_time (EphyHistoryService *self, gint64 from,
 {
   EphyHistoryQuery *query;
 
-  g_return_if_fail (EPHY_IS_HISTORY_SERVICE (self));
+  g_assert (EPHY_IS_HISTORY_SERVICE (self));
 
   query = ephy_history_query_new ();
   query->from = from;
@@ -736,8 +720,8 @@ ephy_history_service_query_visits (EphyHistoryService *self, EphyHistoryQuery *q
 {
   EphyHistoryServiceMessage *message;
 
-  g_return_if_fail (EPHY_IS_HISTORY_SERVICE (self));
-  g_return_if_fail (query != NULL);
+  g_assert (EPHY_IS_HISTORY_SERVICE (self));
+  g_assert (query != NULL);
 
   message = ephy_history_service_message_new (self, QUERY_VISITS,
                                               ephy_history_query_copy (query), (GDestroyNotify)ephy_history_query_free, cancellable, callback, user_data);
@@ -759,8 +743,8 @@ ephy_history_service_query_urls (EphyHistoryService *self, EphyHistoryQuery *que
 {
   EphyHistoryServiceMessage *message;
 
-  g_return_if_fail (EPHY_IS_HISTORY_SERVICE (self));
-  g_return_if_fail (query != NULL);
+  g_assert (EPHY_IS_HISTORY_SERVICE (self));
+  g_assert (query != NULL);
 
   message = ephy_history_service_message_new (self, QUERY_URLS,
                                               ephy_history_query_copy (query), (GDestroyNotify)ephy_history_query_free,
@@ -776,7 +760,7 @@ ephy_history_service_get_hosts (EphyHistoryService    *self,
 {
   EphyHistoryServiceMessage *message;
 
-  g_return_if_fail (EPHY_IS_HISTORY_SERVICE (self));
+  g_assert (EPHY_IS_HISTORY_SERVICE (self));
 
   message = ephy_history_service_message_new (self, GET_HOSTS,
                                               NULL, NULL, cancellable,
@@ -793,7 +777,7 @@ ephy_history_service_query_hosts (EphyHistoryService    *self,
 {
   EphyHistoryServiceMessage *message;
 
-  g_return_if_fail (EPHY_IS_HISTORY_SERVICE (self));
+  g_assert (EPHY_IS_HISTORY_SERVICE (self));
 
   message = ephy_history_service_message_new (self, QUERY_HOSTS,
                                               ephy_history_query_copy (query),
@@ -854,10 +838,10 @@ ephy_history_service_set_url_title (EphyHistoryService    *self,
   EphyHistoryURL *url;
   EphyHistoryServiceMessage *message;
 
-  g_return_if_fail (EPHY_IS_HISTORY_SERVICE (self));
-  g_return_if_fail (orig_url != NULL);
-  g_return_if_fail (title != NULL);
-  g_return_if_fail (*title != '\0');
+  g_assert (EPHY_IS_HISTORY_SERVICE (self));
+  g_assert (orig_url != NULL);
+  g_assert (title != NULL);
+  g_assert (*title != '\0');
 
   url = ephy_history_url_new (orig_url, title, 0, 0, 0);
   message = ephy_history_service_message_new (self, SET_URL_TITLE,
@@ -883,7 +867,7 @@ ephy_history_service_execute_set_url_zoom_level (EphyHistoryService *self,
   host = ephy_history_service_get_host_row_from_url (self, url_string);
   g_free (url_string);
 
-  g_return_val_if_fail (host != NULL, FALSE);
+  g_assert (host != NULL);
 
   host->zoom_level = zoom_level;
   ephy_history_service_update_host_row (self, host);
@@ -902,8 +886,8 @@ ephy_history_service_set_url_zoom_level (EphyHistoryService    *self,
   EphyHistoryServiceMessage *message;
   GVariant *variant;
 
-  g_return_if_fail (EPHY_IS_HISTORY_SERVICE (self));
-  g_return_if_fail (url != NULL);
+  g_assert (EPHY_IS_HISTORY_SERVICE (self));
+  g_assert (url != NULL);
 
   variant = g_variant_new ("(sd)", url, zoom_level);
 
@@ -946,8 +930,8 @@ ephy_history_service_set_url_hidden (EphyHistoryService    *self,
   EphyHistoryServiceMessage *message;
   EphyHistoryURL *url;
 
-  g_return_if_fail (EPHY_IS_HISTORY_SERVICE (self));
-  g_return_if_fail (orig_url != NULL);
+  g_assert (EPHY_IS_HISTORY_SERVICE (self));
+  g_assert (orig_url != NULL);
 
   url = ephy_history_url_new (orig_url, NULL, 0, 0, 0);
   url->hidden = hidden;
@@ -990,8 +974,8 @@ ephy_history_service_set_url_thumbnail_time (EphyHistoryService    *self,
   EphyHistoryURL *url;
   EphyHistoryServiceMessage *message;
 
-  g_return_if_fail (EPHY_IS_HISTORY_SERVICE (self));
-  g_return_if_fail (orig_url != NULL);
+  g_assert (EPHY_IS_HISTORY_SERVICE (self));
+  g_assert (orig_url != NULL);
 
   url = ephy_history_url_new (orig_url, NULL, 0, 0, 0);
   url->thumbnail_time = thumbnail_time;
@@ -1024,8 +1008,8 @@ ephy_history_service_get_url (EphyHistoryService    *self,
 {
   EphyHistoryServiceMessage *message;
 
-  g_return_if_fail (EPHY_IS_HISTORY_SERVICE (self));
-  g_return_if_fail (url != NULL);
+  g_assert (EPHY_IS_HISTORY_SERVICE (self));
+  g_assert (url != NULL);
 
   message = ephy_history_service_message_new (self, GET_URL,
                                               g_strdup (url), g_free,
@@ -1041,7 +1025,7 @@ ephy_history_service_execute_get_host_for_url (EphyHistoryService *self,
   EphyHistoryHost *host;
 
   host = ephy_history_service_get_host_row_from_url (self, url);
-  g_return_val_if_fail (host != NULL, FALSE);
+  g_assert (host != NULL);
 
   *result = host;
 
@@ -1057,8 +1041,8 @@ ephy_history_service_get_host_for_url (EphyHistoryService    *self,
 {
   EphyHistoryServiceMessage *message;
 
-  g_return_if_fail (EPHY_IS_HISTORY_SERVICE (self));
-  g_return_if_fail (url != NULL);
+  g_assert (EPHY_IS_HISTORY_SERVICE (self));
+  g_assert (url != NULL);
 
   message = ephy_history_service_message_new (self, GET_HOST_FOR_URL,
                                               g_strdup (url), g_free,
@@ -1145,22 +1129,13 @@ ephy_history_service_execute_clear (EphyHistoryService *self,
                                     gpointer            pointer,
                                     gpointer           *result)
 {
-  char *journal_filename;
-
   if (self->history_database == NULL ||
       self->read_only)
     return FALSE;
 
   ephy_history_service_commit_transaction (self);
   ephy_sqlite_connection_close (self->history_database);
-
-  if (g_unlink (self->history_filename) == -1)
-    g_warning ("Failed to delete %s: %s", self->history_filename, g_strerror (errno));
-
-  journal_filename = g_strdup_printf ("%s-journal", self->history_filename);
-  if (g_unlink (journal_filename) == -1 && errno != ENOENT)
-    g_warning ("Failed to delete %s: %s", journal_filename, g_strerror (errno));
-  g_free (journal_filename);
+  ephy_sqlite_connection_delete_database (self->history_database);
 
   ephy_history_service_open_database_connections (self);
   ephy_history_service_open_transaction (self);
@@ -1177,8 +1152,8 @@ ephy_history_service_delete_urls (EphyHistoryService    *self,
 {
   EphyHistoryServiceMessage *message;
 
-  g_return_if_fail (EPHY_IS_HISTORY_SERVICE (self));
-  g_return_if_fail (urls != NULL);
+  g_assert (EPHY_IS_HISTORY_SERVICE (self));
+  g_assert (urls != NULL);
 
   message = ephy_history_service_message_new (self, DELETE_URLS,
                                               ephy_history_url_list_copy (urls), (GDestroyNotify)ephy_history_url_list_free,
@@ -1208,7 +1183,7 @@ ephy_history_service_clear (EphyHistoryService    *self,
 {
   EphyHistoryServiceMessage *message;
 
-  g_return_if_fail (EPHY_IS_HISTORY_SERVICE (self));
+  g_assert (EPHY_IS_HISTORY_SERVICE (self));
 
   message = ephy_history_service_message_new (self, CLEAR,
                                               NULL, NULL,
@@ -1299,7 +1274,7 @@ ephy_history_service_find_urls (EphyHistoryService *self,
 {
   EphyHistoryQuery *query;
 
-  g_return_if_fail (EPHY_IS_HISTORY_SERVICE (self));
+  g_assert (EPHY_IS_HISTORY_SERVICE (self));
 
   query = ephy_history_query_new ();
   query->from = from;
@@ -1327,9 +1302,9 @@ ephy_history_service_visit_url (EphyHistoryService       *self,
 {
   EphyHistoryPageVisit *visit;
 
-  g_return_if_fail (EPHY_IS_HISTORY_SERVICE (self));
-  g_return_if_fail (url != NULL);
-  g_return_if_fail (visit_time > 0);
+  g_assert (EPHY_IS_HISTORY_SERVICE (self));
+  g_assert (url != NULL);
+  g_assert (visit_time > 0);
 
   visit = ephy_history_page_visit_new (url, visit_time, visit_type);
   visit->url->sync_id = g_strdup (sync_id);
@@ -1349,7 +1324,7 @@ ephy_history_service_find_hosts (EphyHistoryService *self,
 {
   EphyHistoryQuery *query;
 
-  g_return_if_fail (EPHY_IS_HISTORY_SERVICE (self));
+  g_assert (EPHY_IS_HISTORY_SERVICE (self));
 
   query = ephy_history_query_new ();
 
